@@ -2,17 +2,20 @@
 
 import sys
 import re
+import hashlib
 import lxml.html
 from lxml.html.clean import Cleaner
 import logging
-from urllib.parse import urlparse
+from urllib.parse import urlparse, unquote
 import pprint
 from GoogleScraper.database import SearchEngineResultsPage, Link
+from GoogleScraper.config import Config
+from GoogleScraper.log import out
 from cssselect import HTMLTranslator
 
 logger = logging.getLogger('GoogleScraper')
 
-class InvalidSearchTypeExcpetion(Exception):
+class InvalidSearchTypeException(Exception):
     pass
 
 
@@ -52,7 +55,7 @@ class Parser():
     # If you didn't specify the search type in the search_types list, this attribute
     # will not be evaluated and no data will be parsed.
 
-    def __init__(self, html=None, searchtype='normal'):
+    def __init__(self, html=None):
         """Create new Parser instance and parse all information.
 
         Args:
@@ -64,10 +67,10 @@ class Parser():
             Assertion error if the subclassed
             specific parser cannot handle the the settings.
         """
-        assert searchtype in self.search_types
+        self.searchtype = Config['SCRAPING'].get('search_type', 'normal')
+        assert self.searchtype in self.search_types, 'search type "{}" is not supported in {}'.format(self.searchtype, self.__class__.__name__)
         
         self.html = html
-        self.searchtype = searchtype
         self.dom = None
         self.search_results = {}
         self.search_results['num_results'] = ''
@@ -87,29 +90,27 @@ class Parser():
         self._parse()
         
         # Apply subclass specific behaviour after parsing has happened
+        # This is needed because different parsers need to clean/modify
+        # the parsed data uniquely.
         self.after_parsing()
 
-    def _parse(self):
-        """Internal parse the dom according to the provided css selectors.
-        
-        Raises: InvalidSearchTypeExcpetion if no css selectors for the searchtype could be found.
-        """
-        
-        # Try to parse the provided HTML string using lxml
-        # strip all unnecessary information to save space
-        cleaner = Cleaner()
-        cleaner.scripts = True
-        cleaner.javascript = True
-        cleaner.style = True
-
+    def _parse_lxml(self, cleaner=None):
         try:
             parser = lxml.html.HTMLParser(encoding='utf-8')
+            if cleaner:
+                self.dom = cleaner.clean_html(self.dom)
             self.dom = lxml.html.document_fromstring(self.html, parser=parser)
-            self.dom = cleaner.clean_html(self.dom)
             self.dom.resolve_base_href()
         except Exception as e:
             # maybe wrong encoding
             logger.error(e)
+
+    def _parse(self, cleaner=None):
+        """Internal parse the dom according to the provided css selectors.
+        
+        Raises: InvalidSearchTypeExcpetion if no css selectors for the searchtype could be found.
+        """
+        self._parse_lxml(cleaner)
         
         # try to parse the number of results.
         attr_name = self.searchtype + '_search_selectors'
@@ -132,7 +133,7 @@ class Parser():
                     break
 
         if not selector_dict and not isinstance(selector_dict, dict):
-            raise InvalidSearchTypeExcpetion('There is no such attribute: {}. No selectors found'.format(attr_name))
+            raise InvalidSearchTypeException('There is no such attribute: {}. No selectors found'.format(attr_name))
 
         for result_type, selector_class in selector_dict.items():
 
@@ -143,7 +144,6 @@ class Parser():
                 results = self.dom.xpath(
                     css_to_xpath('{container} {result_container}'.format(**selectors))
                 )
-
                 to_extract = set(selectors.keys()) - {'container', 'result_container'}
                 selectors_to_use = {key: selectors[key] for key in to_extract if key in selectors.keys()}
 
@@ -151,12 +151,14 @@ class Parser():
                     # Let's add primitive support for CSS3 pseudo selectors
                     # We just need two of them
                     # ::text
-                    # ::attr(someattribute)
+                    # ::attr(attribute)
 
-                    # You say we should use xpath expresssions instead?
+                    # You say we should use xpath expressions instead?
                     # Maybe you're right, but they are complicated when it comes to classes,
                     # have a look here: http://doc.scrapy.org/en/latest/topics/selectors.html
                     serp_result = {}
+                    # key are for example 'link', 'snippet', 'visible-url', ...
+                    # selector is the selector to grab these items
                     for key, selector in selectors_to_use.items():
                         value = None
                         if selector.endswith('::text'):
@@ -176,9 +178,30 @@ class Parser():
                                     value = result.xpath(css_to_xpath(selector))[0].text_content()
                                 except IndexError as e:
                                     pass
+
                         serp_result[key] = value
-                    if serp_result:
+                    # only add items that have not None links.
+                    # Avoid duplicates. Detect them by the link.
+                    # If statement below: Lazy evaluation. The more probable case first.
+                    if 'link' in serp_result and serp_result['link'] and \
+                            not [e for e in self.search_results[result_type] if e['link'] == serp_result['link']]:
                         self.search_results[result_type].append(serp_result)
+
+    def result_id(self, args):
+        """Gets an unique id for the args.
+
+        Args:
+            args: A number of arguments that constitute the uniqueness of the result.
+
+        Returns:
+            An hash of the concatenated arguments.
+        """
+        hasher = hashlib.md5()
+        for arg in args:
+            if arg:
+                hasher.update(arg.encode())
+
+        return hasher.hexdigest()
                     
     def after_parsing(self):
         """Subclass specific behaviour after parsing happened.
@@ -188,12 +211,20 @@ class Parser():
         """
                 
     def __str__(self):
-        """Return a nicely formated overview of the results."""
+        """Return a nicely formatted overview of the results."""
         return pprint.pformat(self.search_results)
 
     @property
     def cleaned_html(self):
-        assert self.dom, 'The html needs to be parsed to get the cleaned html'
+        # Try to parse the provided HTML string using lxml
+        # strip all unnecessary information to save space
+        cleaner = Cleaner()
+        cleaner.scripts = True
+        cleaner.javascript = True
+        cleaner.comments = True
+        cleaner.style = True
+        self.dom = cleaner.clean_html(self.dom)
+        assert len(self.dom), 'The html needs to be parsed to get the cleaned html'
         return lxml.html.tostring(self.dom)
                 
 """
@@ -272,11 +303,17 @@ class GoogleParser(Parser):
     }
     
     image_search_selectors = {
-        'de_ip': {
-            'results': {
+        'results': {
+            'de_ip': {
                 'container': 'li#isr_mc',
                 'result_container': 'div.rg_di',
-                'imgurl': 'a.rg_l::attr(href)'
+                'link': 'a.rg_l::attr(href)'
+            },
+            'de_ip_raw': {
+                'container': '.images_table',
+                'result_container': 'tr td',
+                'link': 'a::attr(href)',
+                'visible_link': 'cite::text',
             }
         }
     }
@@ -294,19 +331,28 @@ class GoogleParser(Parser):
         Clean with a short regex.
         """
         super().after_parsing()
+
+        clean_regexes = {
+            'normal': r'/url\?q=(?P<url>.*?)&sa=U&ei=',
+            'image': r'imgres\?imgurl=(?P<url>.*?)&'
+        }
+
         for key, value in self.search_results.items():
             if isinstance(value, list):
                 for i, item in enumerate(value):
                     if isinstance(item, dict) and item['link']:
-                        result = re.search(r'/url\?q=(?P<url>.*?)&sa=U&ei=', item['link'])
+                        result = re.search(
+                            clean_regexes[self.searchtype],
+                            item['link']
+                        )
                         if result:
-                            self.search_results[key][i]['link'] = result.group('url')
-                            
+                            self.search_results[key][i]['link'] = unquote(result.group('url'))
+
 
 class YandexParser(Parser):
     """Parses SERP pages of the Yandex search engine."""
 
-    search_types = ['normal']
+    search_types = ['normal', 'image']
     
     num_results_search_selectors = []
     
@@ -322,12 +368,57 @@ class YandexParser(Parser):
             }
         }
     }
+
+    image_search_selectors = {
+        'results': {
+            'de_ip': {
+                'container': '.page-layout__content-wrapper',
+                'result_container': '.serp-item__preview',
+                'link': '.serp-item__preview .serp-item__link::attr(onmousedown)'
+            },
+            'de_ip_raw': {
+                'container': '.page-layout__content-wrapper',
+                'result_container': '.serp-item__preview',
+                'link': '.serp-item__preview .serp-item__link::attr(href)'
+            }
+        }
+    }
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    def after_parsing(self):
+        """Clean the urls.
+
+        Normally Yandex image search store the image url in the onmousedown attribute in a json object. Its
+        pretty messsy. This method grabs the link with a quick regex.
+
+        c.hit({"dtype":"iweb","path":"8.228.471.241.184.141","pos":69,"reqid":"1418919408668565-676535248248925882431999-ws35-986-IMG-p2"}, {"href":"http://www.thewallpapers.org/wallpapers/3/382/thumb/600_winter-snow-nature002.jpg"});
+
+        Sometimes the img url is also stored in the href attribute (when requesting with raw http packets).
+        href="/images/search?text=snow&img_url=http%3A%2F%2Fwww.proza.ru%2Fpics%2F2009%2F12%2F07%2F1290.jpg&pos=2&rpt=simage&pin=1">
+        """
+        super().after_parsing()
+
+        if self.searchtype == 'image':
+            for key, value in self.search_results.items():
+                if isinstance(value, list):
+                    for i, item in enumerate(value):
+                        if isinstance(item, dict) and item['link']:
+                            for regex in (
+                                r'\{"href"\s*:\s*"(?P<url>.*?)"\}',
+                                r'img_url=(?P<url>.*?)&'
+                            ):
+                                result = re.search(regex, item['link'])
+                                if result:
+                                    self.search_results[key][i]['link'] = result.group('url')
+                                    break
     
     
 class BingParser(Parser):
     """Parses SERP pages of the Bing search engine."""
     
-    search_types = ['normal']
+    search_types = ['normal', 'image']
     
     num_results_search_selectors = ['.sb_count']
     
@@ -370,11 +461,49 @@ class BingParser(Parser):
         }
     }
 
+    image_search_selectors = {
+        'results': {
+            'ch_ip': {
+                'container': '#dg_c .imgres',
+                'result_container': '.dg_u',
+                'link': 'a.dv_i::attr(m)'
+            },
+        }
+    }
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+
+    def after_parsing(self):
+        """Clean the urls.
+
+        The image url data is in the m attribute.
+
+        m={ns:"images.1_4",k:"5018",mid:"46CE8A1D71B04B408784F0219B488A5AE91F972E",
+        surl:"http://berlin-germany.ca/",imgurl:"http://berlin-germany.ca/images/berlin250.jpg",
+        oh:"184",tft:"45",oi:"http://berlin-germany.ca/images/berlin250.jpg"}
+        """
+        super().after_parsing()
+
+        if self.searchtype == 'image':
+            for key, value in self.search_results.items():
+                if isinstance(value, list):
+                    for i, item in enumerate(value):
+                        if isinstance(item, dict) and item['link']:
+                            for regex in (
+                                r'imgurl:"(?P<url>.*?)"',
+                            ):
+                                result = re.search(regex, item['link'])
+                                if result:
+                                    self.search_results[key][i]['link'] = result.group('url')
+                                    break
+
 
 class YahooParser(Parser):
     """Parses SERP pages of the Yahoo search engine."""
     
-    search_types = ['normal']
+    search_types = ['normal', 'image']
     
     num_results_search_selectors = ['#pg > span:last-child']
     
@@ -390,12 +519,54 @@ class YahooParser(Parser):
             }
         },
     }
-    
+
+    image_search_selectors = {
+        'results': {
+            'ch_ip': {
+                'container': '#results',
+                'result_container': '#sres > li',
+                'link': 'a::attr(href)'
+            },
+        }
+    }
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+
+    def after_parsing(self):
+        """Clean the urls.
+
+        The url is in the href attribute and the &imgurl= parameter.
+
+        <a id="yui_3_5_1_1_1419284335995_1635" aria-label="<b>Matterhorn</b> sunrise"
+        href="/images/view;_ylt=AwrB8phvj5hU7moAFzOJzbkF;_ylu=X3oDMTIyc3ZrZ3RwBHNlYwNzcgRzbGsDaW1nBG9pZANmNTgyY2MyYTY4ZmVjYTI5YmYwNWZlM2E3ZTc1YzkyMARncG9zAzEEaXQDYmluZw--?
+        .origin=&back=https%3A%2F%2Fimages.search.yahoo.com%2Fsearch%2Fimages%3Fp%3Dmatterhorn%26fr%3Dyfp-t-901%26fr2%3Dpiv-web%26tab%3Dorganic%26ri%3D1&w=4592&h=3056&
+        imgurl=www.summitpost.org%2Fimages%2Foriginal%2F699696.JPG&rurl=http%3A%2F%2Fwww.summitpost.org%2Fmatterhorn-sunrise%2F699696&size=5088.0KB&
+        name=%3Cb%3EMatterhorn%3C%2Fb%3E+sunrise&p=matterhorn&oid=f582cc2a68feca29bf05fe3a7e75c920&fr2=piv-web&
+        fr=yfp-t-901&tt=%3Cb%3EMatterhorn%3C%2Fb%3E+sunrise&b=0&ni=21&no=1&ts=&tab=organic&
+        sigr=11j056ue0&sigb=134sbn4gc&sigi=11df3qlvm&sigt=10pd8j49h&sign=10pd8j49h&.crumb=qAIpMoHvtm1&fr=yfp-t-901&fr2=piv-web">
+        """
+        super().after_parsing()
+
+        if self.searchtype == 'image':
+            for key, value in self.search_results.items():
+                if isinstance(value, list):
+                    for i, item in enumerate(value):
+                        if isinstance(item, dict) and item['link']:
+                            for regex in (
+                                r'&imgurl=(?P<url>.*?)&',
+                            ):
+                                result = re.search(regex, item['link'])
+                                if result:
+                                    # TODO: Fix this manual protocol adding by parsing "rurl"
+                                    self.search_results[key][i]['link'] = 'http://' + unquote(result.group('url'))
+                                    break
 
 class BaiduParser(Parser):
     """Parses SERP pages of the Baidu search engine."""
     
-    search_types = ['normal']
+    search_types = ['normal', 'image']
     
     num_results_search_selectors = ['#container .nums']
     
@@ -412,11 +583,48 @@ class BaiduParser(Parser):
         },
     }
 
+    image_search_selectors = {
+        'results': {
+            'ch_ip': {
+                'container': '#imgContainer',
+                'result_container': '.pageCon > li',
+                'link': '.imgShow a::attr(href)'
+            },
+        }
+    }
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+
+    def after_parsing(self):
+        """Clean the urls.
+
+        href="/i?ct=503316480&z=&tn=baiduimagedetail&ipn=d&word=matterhorn&step_word=&ie=utf-8&in=9250&
+        cl=2&lm=-1&st=&cs=3326243323,1574167845&os=1495729451,4260959385&pn=0&rn=1&di=69455168860&ln=1285&
+        fr=&&fmq=1419285032955_R&ic=&s=&se=&sme=0&tab=&width=&height=&face=&is=&istype=&ist=&jit=&
+        objurl=http%3A%2F%2Fa669.phobos.apple.com%2Fus%2Fr1000%2F077%2FPurple%2Fv4%2F2a%2Fc6%2F15%2F2ac6156c-e23e-62fd-86ee-7a25c29a6c72%2Fmzl.otpvmwuj.1024x1024-65.jpg&adpicid=0"
+        """
+        super().after_parsing()
+
+        if self.searchtype == 'image':
+            for key, value in self.search_results.items():
+                if isinstance(value, list):
+                    for i, item in enumerate(value):
+                        if isinstance(item, dict) and item['link']:
+                            for regex in (
+                                r'&objurl=(?P<url>.*?)&',
+                            ):
+                                result = re.search(regex, item['link'])
+                                if result:
+                                    self.search_results[key][i]['link'] = unquote(result.group('url'))
+                                    break
+
 
 class DuckduckgoParser(Parser):
     """Parses SERP pages of the Duckduckgo search engine."""
     
-    search_types = ['normal']
+    search_types = ['normal', 'image']
     
     num_results_search_selectors = []
     
@@ -512,6 +720,7 @@ def parse_serp(html=None, search_engine=None,
             parser = parser()
             parser.parse(html)
 
+        out(parser, lvl=2)
         num_results = 0
 
         if not serp:
@@ -530,6 +739,10 @@ def parse_serp(html=None, search_engine=None,
                 rank = 1
                 for link in value:
                     parsed = urlparse(link['link'])
+
+                    # fill with nones to prevent key errors
+                    [link.update({key: None}) for key in ('snippet', 'title', 'visible_link') if key not in link]
+
                     l = Link(
                         link=link['link'],
                         snippet=link['snippet'],
