@@ -882,35 +882,36 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         Returns:
             The href attribute of the next_url for further results.
         """
+        if self.search_type == 'normal':
+            next_page_selectors = {
+                'google': '#pnnext',
+                'yandex': '.pager__button_kind_next',
+                'bing': '.sb_pagN',
+                'yahoo': '#pg-next',
+                'baidu': '.n',
+                'duckduckgo': '' # loads results dynamically with ajax
+            }
 
-        next_page_selectors = {
-            'google': '#pnnext',
-            'yandex': '.pager__button_kind_next',
-            'bing': '.sb_pagN',
-            'yahoo': '#pg-next',
-            'baidu': '.n',
-            'duckduckgo': '' # loads results dynamically with ajax
-        }
+            selector = next_page_selectors[self.search_engine]
+            next_url = ''
+            try:
+                # wait until the next page link emerges
+                WebDriverWait(self.webdriver, 5).until(EC.presence_of_element_located((By.CSS_SELECTOR, selector)))
+                element = self.webdriver.find_element_by_css_selector(selector)
+                next_url = element.get_attribute('href')
+                element.click()
+                self.current_request_time = datetime.datetime.utcnow()
+            except TimeoutException as te:
+                logger.warning('Cannot locate next page element: {}'.format(te))
+                return False
+            except WebDriverException as e:
+                logger.warning('Cannot locate next page element: {}'.format(e))
+                return False
 
-        selector = next_page_selectors[self.search_engine]
-
-        next_url = ''
-
-        try:
-            # wait until the next page link emerges
-            WebDriverWait(self.webdriver, 5).until(EC.presence_of_element_located((By.CSS_SELECTOR, selector)))
-            element = self.webdriver.find_element_by_css_selector(selector)
-            next_url = element.get_attribute('href')
-            element.click()
-            self.current_request_time = datetime.datetime.utcnow()
-        except TimeoutException as te:
-            logger.warning('Cannot locate next page element: {}'.format(te))
-            return False
-        except WebDriverException as e:
-            logger.warning('Cannot locate next page element: {}'.format(e))
-            return False
-
-        return next_url
+            return next_url
+        elif self.search_type == 'image':
+            self.page_down()
+            return True
 
     def search(self):
         """Search with webdriver.
@@ -958,6 +959,27 @@ class SelScrape(SearchEngineScrape, threading.Thread):
 
             n += 1
 
+    def page_down(self):
+        """Scrolls down a page with javascript.
+
+        Used for next page in image search mode.
+        """
+        js = '''
+        var B = document.body,
+            H = document.documentElement,
+            height
+
+        if (typeof document.height !== 'undefined') {
+            height = document.height // For webkit browsers
+        } else {
+            height = Math.max( B.scrollHeight, B.offsetHeight,H.clientHeight, H.scrollHeight, H.offsetHeight );
+        }
+
+        window.scrollBy(0,800)
+        '''
+
+        self.webdriver.execute_script(js)
+        time.sleep(1.5)
 
     def run(self):
         """Run the SelScraper."""
